The files in this directory are intended to be used with the --plan option of
telegraph send.

* spark2014.plan: identical to the spark2014 preset
* spark2014+gnat.plan: identical to the spark2014-ext preset
* spark2014_only.plan: identical to the spark2014 preset, without build of why3
