Victor: A SPARK Verification Condition Translator and Prover Driver.

Version 0.9.1
15th December 2010

Paul Jackson
pbj@inf.ed.ac.uk


See section "Installation and Testing" of the User Manual
doc/vct-man.pdf for information on how to install and run Victor.


Distribution organisation:

README.txt   # This file
CHANGES.txt  # Changes made in this and earlier releases.
COPYING.txt  # copyright and licensing information
LICENSE.txt  # GNU GPL V3 license

doc/vct-man.pdf # User manual.  

src/        # Source code for Victor and auxiliary tools
  build/    # Example intermediate source files and executables

run/        # Makefile for running Victor and various input files
  out-ref/  # Example report files.

vc/   # Example VC sets

bin/  # Default location for executables built from sources. 
        Empty in distribution.
