#
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# zzz.R - package startup message

.onAttach <- function(...) {
  if (Sys.getenv("SPARKR_SUPPRESS_DEPRECATION_WARNING") == "") {
    packageStartupMessage(
      paste0(
        "Warning: ",
        "SparkR is deprecated in Apache Spark 4.0.0 and will be removed in a future release. ",
        "To continue using Spark in R, we recommend using sparklyr instead: ",
        "https://spark.posit.co/get-started/"
      )
    )
  }
}
