| source \ target                                     | list<item: int32>                              | list<item: int64>                              | list<item: string>                                  | large_list<item: int32>                              | large_list<item: int64>                              | fixed_size_list<item: int32>[2]                        | fixed_size_list<item: int32>[3]                   | map<string, int32>                              | map<string, int64>                              | struct<x: int32, y: string>                                    | struct<x: int64, y: string>                                    | struct<y: string, x: int32>                                    | struct<a: int32, b: string>                                    | string                       | int32                        |
|-----------------------------------------------------|------------------------------------------------|------------------------------------------------|-----------------------------------------------------|------------------------------------------------------|------------------------------------------------------|--------------------------------------------------------|---------------------------------------------------|-------------------------------------------------|-------------------------------------------------|----------------------------------------------------------------|----------------------------------------------------------------|----------------------------------------------------------------|----------------------------------------------------------------|------------------------------|------------------------------|
| list<item: int32>:standard                          | [[1, 2, 3], None]@list<item: int32>            | [[1, 2, 3], None]@list<item: int64>            | [['1', '2', '3'], None]@list<item: string>          | [[1, 2, 3], None]@large_list<item: int32>            | [[1, 2, 3], None]@large_list<item: int64>            | ERR@ArrowInvalid                                       | [[1, 2, 3], None]@fixed_size_list<item: int32>[3] | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError | ERR@ArrowNotImplementedError |
| list<item: int32>:empty                             | [[], None]@list<item: int32>                   | [[], None]@list<item: int64>                   | [[], None]@list<item: string>                       | [[], None]@large_list<item: int32>                   | [[], None]@large_list<item: int64>                   | ERR@ArrowInvalid                                       | ERR@ArrowInvalid                                  | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError | ERR@ArrowNotImplementedError |
| list<item: int32>:null_elem                         | [[None], [1, None, 3], None]@list<item: int32> | [[None], [1, None, 3], None]@list<item: int64> | [[None], ['1', None, '3'], None]@list<item: string> | [[None], [1, None, 3], None]@large_list<item: int32> | [[None], [1, None, 3], None]@large_list<item: int64> | ERR@ArrowInvalid                                       | ERR@ArrowInvalid                                  | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError | ERR@ArrowNotImplementedError |
| list<item: struct<x: int32, y: string>>:standard    | ERR@ArrowNotImplementedError                   | ERR@ArrowNotImplementedError                   | ERR@ArrowNotImplementedError                        | ERR@ArrowNotImplementedError                         | ERR@ArrowNotImplementedError                         | ERR@ArrowInvalid                                       | ERR@ArrowInvalid                                  | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError | ERR@ArrowNotImplementedError |
| list<item: struct<x: int32, y: string>>:null_fields | ERR@ArrowNotImplementedError                   | ERR@ArrowNotImplementedError                   | ERR@ArrowNotImplementedError                        | ERR@ArrowNotImplementedError                         | ERR@ArrowNotImplementedError                         | ERR@ArrowInvalid                                       | ERR@ArrowInvalid                                  | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError | ERR@ArrowNotImplementedError |
| large_list<item: int32>:standard                    | [[1, 2, 3], None]@list<item: int32>            | [[1, 2, 3], None]@list<item: int64>            | [['1', '2', '3'], None]@list<item: string>          | [[1, 2, 3], None]@large_list<item: int32>            | [[1, 2, 3], None]@large_list<item: int64>            | ERR@ArrowInvalid                                       | [[1, 2, 3], None]@fixed_size_list<item: int32>[3] | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError | ERR@ArrowNotImplementedError |
| large_list<item: int32>:empty                       | [[], None]@list<item: int32>                   | [[], None]@list<item: int64>                   | [[], None]@list<item: string>                       | [[], None]@large_list<item: int32>                   | [[], None]@large_list<item: int64>                   | ERR@ArrowInvalid                                       | ERR@ArrowInvalid                                  | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError | ERR@ArrowNotImplementedError |
| large_list<item: int32>:null_elem                   | [[None], None]@list<item: int32>               | [[None], None]@list<item: int64>               | [[None], None]@list<item: string>                   | [[None], None]@large_list<item: int32>               | [[None], None]@large_list<item: int64>               | ERR@ArrowInvalid                                       | ERR@ArrowInvalid                                  | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError | ERR@ArrowNotImplementedError |
| fixed_size_list<item: int32>[2]:standard            | [[1, 2], [3, 4], None]@list<item: int32>       | [[1, 2], [3, 4], None]@list<item: int64>       | [['1', '2'], ['3', '4'], None]@list<item: string>   | [[1, 2], [3, 4], None]@large_list<item: int32>       | [[1, 2], [3, 4], None]@large_list<item: int64>       | [[1, 2], [3, 4], None]@fixed_size_list<item: int32>[2] | ERR@ArrowTypeError                                | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError | ERR@ArrowNotImplementedError |
| fixed_size_list<item: int32>[2]:null_elem           | [[None, None], None]@list<item: int32>         | [[None, None], None]@list<item: int64>         | [[None, None], None]@list<item: string>             | [[None, None], None]@large_list<item: int32>         | [[None, None], None]@large_list<item: int64>         | [[None, None], None]@fixed_size_list<item: int32>[2]   | ERR@ArrowTypeError                                | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError | ERR@ArrowNotImplementedError |
| map<string, int32>:standard                         | ERR@ArrowTypeError                             | ERR@ArrowTypeError                             | ERR@ArrowTypeError                                  | ERR@ArrowTypeError                                   | ERR@ArrowTypeError                                   | ERR@ArrowNotImplementedError                           | ERR@ArrowInvalid                                  | [[('a', 1), ('b', 2)], None]@map<string, int32> | [[('a', 1), ('b', 2)], None]@map<string, int64> | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError | ERR@ArrowNotImplementedError |
| map<string, int32>:empty                            | ERR@ArrowTypeError                             | ERR@ArrowTypeError                             | ERR@ArrowTypeError                                  | ERR@ArrowTypeError                                   | ERR@ArrowTypeError                                   | ERR@ArrowInvalid                                       | ERR@ArrowInvalid                                  | [[], None]@map<string, int32>                   | [[], None]@map<string, int64>                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError                                   | ERR@ArrowNotImplementedError | ERR@ArrowNotImplementedError |
| struct<x: int32, y: string>:standard                | ERR@ArrowNotImplementedError                   | ERR@ArrowNotImplementedError                   | ERR@ArrowNotImplementedError                        | ERR@ArrowNotImplementedError                         | ERR@ArrowNotImplementedError                         | ERR@ArrowNotImplementedError                           | ERR@ArrowNotImplementedError                      | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                    | [[('x', 1), ('y', 'a')], None]@struct<x: int32, y: string>     | [[('x', 1), ('y', 'a')], None]@struct<x: int64, y: string>     | [[('y', 'a'), ('x', 1)], None]@struct<y: string, x: int32>     | [[('a', None), ('b', None)], None]@struct<a: int32, b: string> | ERR@ArrowNotImplementedError | ERR@ArrowNotImplementedError |
| struct<x: int32, y: string>:null_fields             | ERR@ArrowNotImplementedError                   | ERR@ArrowNotImplementedError                   | ERR@ArrowNotImplementedError                        | ERR@ArrowNotImplementedError                         | ERR@ArrowNotImplementedError                         | ERR@ArrowNotImplementedError                           | ERR@ArrowNotImplementedError                      | ERR@ArrowNotImplementedError                    | ERR@ArrowNotImplementedError                    | [[('x', None), ('y', None)], None]@struct<x: int32, y: string> | [[('x', None), ('y', None)], None]@struct<x: int64, y: string> | [[('y', None), ('x', None)], None]@struct<y: string, x: int32> | [[('a', None), ('b', None)], None]@struct<a: int32, b: string> | ERR@ArrowNotImplementedError | ERR@ArrowNotImplementedError |