| SQL Type \ Pandas Value(Type)   | None@NoneType   | True@bool   | 1@int   | a@str   | 1970-01-01@date                          | 1970-01-01 00:00:00@datetime             | 1.0@float   | array('i', [1])@array   | [1]@list   | (1,)@tuple                   | bytearray(b'ABC')@bytearray   | 1@Decimal    | {'a': 1}@dict   | Row(kwargs=1)@Row   | Row(namedtuple=1)@Row   |
|---------------------------------|-----------------|-------------|---------|---------|------------------------------------------|------------------------------------------|-------------|-------------------------|------------|------------------------------|-------------------------------|--------------|-----------------|---------------------|-------------------------|
| boolean                         | None            | True        | None    | None    | None                                     | None                                     | None        | None                    | None       | None                         | None                          | None         | None            | X                   | X                       |
| tinyint                         | None            | None        | 1       | None    | None                                     | None                                     | None        | None                    | None       | None                         | None                          | None         | None            | X                   | X                       |
| smallint                        | None            | None        | 1       | None    | None                                     | None                                     | None        | None                    | None       | None                         | None                          | None         | None            | X                   | X                       |
| int                             | None            | None        | 1       | None    | None                                     | None                                     | None        | None                    | None       | None                         | None                          | None         | None            | X                   | X                       |
| bigint                          | None            | None        | 1       | None    | None                                     | None                                     | None        | None                    | None       | None                         | None                          | None         | None            | X                   | X                       |
| string                          | None            | 'true'      | '1'     | 'a'     | 'java.util.GregorianCalendar[time=?,areF | 'java.util.GregorianCalendar[time=?,areF | '1.0'       | '[I@<hash>'             | '[1]'      | '[Ljava.lang.Object;@<hash>' | '[B@<hash>'                   | '1'          | '{a=1}'         | X                   | X                       |
| date                            | None            | X           | X       | X       | datetime.date(1970, 1, 1)                | datetime.date(1970, 1, 1)                | X           | X                       | X          | X                            | X                             | X            | X               | X                   | X                       |
| timestamp                       | None            | X           | X       | X       | X                                        | datetime.datetime(1970, 1, 1, 0, 0)      | X           | X                       | X          | X                            | X                             | X            | X               | X                   | X                       |
| float                           | None            | None        | None    | None    | None                                     | None                                     | 1.0         | None                    | None       | None                         | None                          | None         | None            | X                   | X                       |
| double                          | None            | None        | None    | None    | None                                     | None                                     | 1.0         | None                    | None       | None                         | None                          | None         | None            | X                   | X                       |
| array<int>                      | None            | None        | None    | None    | None                                     | None                                     | None        | [1]                     | [1]        | [1]                          | [65, 66, 67]                  | None         | None            | X                   | X                       |
| binary                          | None            | None        | None    | b'a'    | None                                     | None                                     | None        | None                    | None       | None                         | b'ABC'                        | None         | None            | X                   | X                       |
| decimal(10,0)                   | None            | None        | None    | None    | None                                     | None                                     | None        | None                    | None       | None                         | None                          | Decimal('1') | None            | X                   | X                       |
| map<string,int>                 | None            | None        | None    | None    | None                                     | None                                     | None        | None                    | None       | None                         | None                          | None         | {'a': 1}        | X                   | X                       |
| struct<_1:int>                  | None            | X           | X       | X       | X                                        | X                                        | X           | X                       | Row(_1=1)  | Row(_1=1)                    | X                             | X            | Row(_1=None)    | Row(_1=1)           | Row(_1=1)               |